%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%          Active Magentic Regenerative Refrigerator Model          %%
%%          Developed at the University of Wisconsin - Madison       %%
%%              Calls Regenerator Model Version 4_2                  %%
%%                                                                   %%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% This input module takes modeling inputs including fluid flow and magnetic
% field profiles and passes them to the regenerator model.  Fluid flow rate
% and magnetic field are determined using functions specified in this
% script.  The outputs of the regenerator model include the temperature
% profile of the fluid and regnerator once cyclical steady state has been
% achieved as well as pumping power and heating and cooling loads.

%% Model inputs
bed_vol=Volume/nbed;            %m^3,volume per bed
D=(4*bed_vol/(AR*pi))^(1/3);    %m, diameter of flow channel
Ac=pi*D^2/4;                    %m^2, cross sectional area of the regenerator
L=AR*D;                         %m, length of each regenerator bed
unbal=0.0;                      %flow unbalance in regenerator
rhof=1000;                      %kg/m^3, density of heat transfer fluid  (water)

cnk=1;                          %axial conduction scaling factor
cff=1;                          %friction factor scaling factor
cNu=1;                           %Nusselt number scaling factor

pacRAW=[0.36,0,0.0002,0];

%inputs associated with the magnetic field profile
maginp=[90,90,360/nbed,2,L];%arc of magnet, arc of ramp, arc of each bed, bed type, regenerator length
%% User-defined functions called to determine properties

fluidfun=@props_water; %the fluid is water
fluidfun2=@props_water2;

bedcorrfun=@SphPart2; %regenerator correlation function

muoHfun=@muoH_ramp2; %ramped field function
muoHfun2=@muoH_ramp22;

mcmpropfun=@JF1142A_nh;%Gd94Er6;%JF1142A; %magnetocaloric material property function
if v==1
    mcmpropfun2=@JF1142A_nh2;  % without magnetic hysteresis
elseif v==2
    mcmpropfun2=@JF1142A2;    % with magnetic hysteresis
end
impmcmpropfun=@JF1142A_nh;%JF1142A_nh%Gd94Er6;    %magnetocaloric property function used for implicit model

massflowfun=@mdotsmoothunbal_delay; %fluid flow function
massflowfun2=@mdotsmoothunbal_delay2;

%% determine mass flow and magnetic field profiles
j=0:Nt-1;
t=j'*tau/(Nt-1);

ii=1:Nx;
jj=0:Nt;
t_node=(jj(1:Nt)'+0.5)/Nt*tau; %time grid at center of time step
xr=L*(ii'-1/2)/Nx; %regenerator spatial grid
delay=(1-dr)*.45*tau; %sec, delay time in seconds from the beginning of the cycle before the blow period begins

massflow_inp=[dr,delay,flow_ramp,unbal]; %inputs associated with the fluid mass flow function
mdot=massflowfun(t,tau,mdot_amp,massflow_inp); %determines the mass flow of fluid vs time
muoH=muoHfun(xr,t,tau,muoH_max,maginp); %determines magnetic field vs time
muoH(:,Nt+2)=muoH(:,1); %fills beginning of next cycle muoH at end of muoH array

%% Bed Layering
TCurie=TH-(TH-TC)*(ceil(ii'-0.5)-0.5)/Nx;  %K, Curie Temperature vector, assigns a T_c to each node