%%  This function provides guess temperatures for the explicit model by
%%  calculating the steady state for temperature profile with the implicit
%%  UW model.

%%  Version 2 is modified to use the regenerator correlations in terms of
%%      hydraulic diameter and other general packing parameters.
%%  Version 3 accepts massflowfun as an input along with massflow_inp
%%      which is an input array for the mass flow rate function
%%  Version 4 uses the new version SphPart2 that corrects for heat transfer
%%  as a function of Bi and Fo
%% Version 5 accepts mdot and muoH profiles as passed variables instead of
%% calculating them using passed functions and function inputs.  Cleaned up
%% unused lines in code.
%% Version 6 is modified to use new bed correlation functions

function[Trn,Tfn,tf,tr]=AMR_impguessT_6(magpropfun,fluidfun,bedcorrfun,pacCOOKED,L,tau,Ac,n,mdot_amp,t_main,TH,TC,TCurie,n_beds,mdot_main,muoH_main);

%% parse pacCooked
%   pacCOOKED=[pacRAW,as,eps,dh,cNu,cff,cnk];

as=pacCOOKED(5);
eps=pacCOOKED(6);
dh=pacCOOKED(7);
cNu=pacCOOKED(8);

m=240; %number of time steps
reltol=0.002; %K, convergence tolerance
wt=1; %weighting function
N=2*m*n+m+n;
A=spalloc(N,N,4*N);
B=spalloc(N,1,N);
modelcheck=1; %1=show T-s and enthalpy flux graphs 0=don't show them

i=0:n;
xf=L*i'/n;
j=1:m;
tf=(j'-0.5)*tau/m;

%%interpolate the mass flow rate passed from main function

mdot=interp1(t_main,mdot_main,tf);

%%

i=1:n;
xr=(i'-0.5)*L/n';
j=0:m;
tr=j'*tau/m;

%% interpolate magnetic field passed from main function

[mu1 mu2]=size(muoH_main);
muoH_middle=interp1(t_main,muoH_main(floor(n/2),1:(mu2-2)),tr); %interpolate to get the magnetic field at the center of the regenerator
muoH_middle(1)=0;%set magnetic field at beginning of cycle to 0
muoH_middle(m+1)=0;%set magnetic field at end of cycle to 0
for i=1:n
    muoH(i,:)=muoH_middle; %assume the magnetic field is spatially uniform and equal to the field at the center of the regenerator
end

%%

for j=0:m
   Trg(:,j+1)=TH-xr*(TH-TC)/L;
end
Trl=Trg;
for j=1:m
 	Tfg(:,j)=TH-xf*(TH-TC)/L;   
end
Tfl=Tfg;

done=0;
while(done==0)
   [muf,kf,cf,rhof,hf,sf]=fluidfun(Tfg);
   [cmuoH,dsdmuoH,rhor,kr,sr,hr,vM_irr]=magpropfun(Trg,TCurie,muoH);
                      %dsdmuoH=ones(n,m+1)*0.0001;
   for i=1:n
      mufn(i,:)=(muf(i,:)+muf(i+1,:))/2;
      cfn(i,:)=(cf(i,:)+cf(i+1,:))/2;
      kfn(i,:)=(kf(i,:)+kf(i+1,:))/2;
   end
   
   v=mdot/(Ac*rhof);
   Prf=cfn.*mufn./(kfn);
   Ref=4*ones(n,1)*abs(v').*rhof./(as*mufn); %based on hydraulic diameter
   
   for j=1:m
   	cmuoH(:,j)=(cmuoH(:,j)+cmuoH(:,j+1))/2;
   	dsdmuoH(:,j)=(dsdmuoH(:,j)+dsdmuoH(:,j+1))/2;
    kr(:,j)=(kr(:,j)+kr(:,j+1))/2;
   end
   kr=kr(floor(n/2),floor(m/2));
   rhor=rhor(floor(n/2),1);
   cf_av=mean(mean(cf));
   cmuoH_av=mean(mean(cmuoH));
   %R=(rhof*eps*cf_av)/(rhor*(1-eps)*cmuoH_av);
   %AF=1+1.764*R+1.0064*R^2;
   %pacCOOKED(8)=cNu*AF;%add lumped heat capacitance correction factor
   [Nuf,ff,keff,kdisp]=bedcorrfun(pacCOOKED,Ref,Prf,kfn,kr,v,muf,rhof,cfn,tau);
   Bi=(Nuf.*kfn)/(2*kr);

   for j=1:m
      if(mdot(j)>=0)
         %hot-to-cold flow
         i=1:n;
         A(((n+1)*(j-1)+i+1-1)*N+(n+1)*(j-1)+i+1)=Nuf(:,j).*kfn(:,j)*as*Ac./((1+Bi(:,j)/5)*(2*dh))+n*mdot(j)*cfn(:,j)/L;
         A(((n+1)*(j-1)+i-1+1-1)*N+(n+1)*(j-1)+i+1)=Nuf(:,j).*kfn(:,j)*as*Ac./((1+Bi(:,j)/5)*(2*dh))-n*mdot(j)*cfn(:,j)/L;   
         A(((n+1)*m+n*j+i-1)*N+(n+1)*(j-1)+i+1)=-Nuf(:,j).*kfn(:,j)*as*Ac./((1+Bi(:,j)/5)*(2*dh));
         A(((n+1)*m+n*(j-1)+i-1)*N+(n+1)*(j-1)+i+1)=-Nuf(:,j).*kfn(:,j)*as*Ac./((1+Bi(:,j)/5)*(2*dh));
         B((n+1)*(j-1)+i+1,1)=ff(:,j).*(v(j)).^2/(2*dh)*rhof*abs(mdot(j))*L/n; %Viscous dissipation, abs(ff(:,j).*(Ref(:,j)).^2*2/dh^3.*(mufn(:,j)/rhof).^2
                             
         i=0;
         A((n+1)*(j-1)+i+1,(n+1)*(j-1)+i+1)=1;
         B((n+1)*(j-1)+i+1,1)=TH;

      else
         if(mdot(j)<0)
            %cold-to-hot flow
	         i=0:(n-1);
             A(((n+1)*(j-1)+i+1-1)*N+(n+1)*(j-1)+i+1)=Nuf(:,j).*kfn(:,j)*as*Ac./((1+Bi(:,j)/5)*(2*dh))-mdot(j)*cfn(:,j)*n/L;
             A(((n+1)*(j-1)+i+1+1-1)*N+(n+1)*(j-1)+i+1)=Nuf(:,j).*kfn(:,j)*as*Ac./((1+Bi(:,j)/5)*(2*dh))+mdot(j)*cfn(:,j)*n/L;
             A(((n+1)*m+n*j+i+1-1)*N+(n+1)*(j-1)+i+1)=-Nuf(:,j).*kfn(:,j)*as*Ac./((1+Bi(:,j)/5)*(2*dh));
             A(((n+1)*m+n*(j-1)+i+1-1)*N+(n+1)*(j-1)+i+1)=-Nuf(:,j).*kfn(:,j)*as*Ac./((1+Bi(:,j)/5)*(2*dh));
             B((n+1)*(j-1)+i+1,1)=ff(:,j).*(v(j)).^2/(2*dh)*rhof*abs(mdot(j))*L/n; %viscous dissipation, abs(ff(:,j).*(Ref(:,j)).^2*2/dh^3.*(mufn(:,j)/rhof).^2
             i=n;
             A((n+1)*(j-1)+i+1,(n+1)*(j-1)+i+1)=1;
             B((n+1)*(j-1)+i+1,1)=TC;
         else
            %no flow
            
         end
      end
   end
   for j=1:m
      i=1:n;
      A(((n+1)*(j-1)+i+1-1)*N+(n+1)*m+n*j+i)=-Nuf(:,j).*kfn(:,j)*as*Ac./((1+Bi(:,j)/5)*(2*dh));
      A(((n+1)*(j-1)+i-1+1-1)*N+(n+1)*m+n*j+i)=-Nuf(:,j).*kfn(:,j)*as*Ac./((1+Bi(:,j)/5)*(2*dh));
      B((n+1)*m+n*j+i,1)=-Ac*(1-eps)*rhor*((Trg(:,j+1)+Trg(:,j))/2).*dsdmuoH(:,j).*(muoH(:,j+1)-muoH(:,j))*m/tau;%...
          %+Ac*(1-eps)*rhor.*vM_irr(:,j).*(muoH(:,j+1)-muoH(:,j))*m/tau;   
      i=2:(n-1);
      A(((n+1)*m+n*j+i-1)*N+(n+1)*m+n*j+i)=Nuf(i',j).*kfn(i',j)*as*Ac./((1+Bi(i',j)/5)*(2*dh))+Ac*(rhof*eps*cfn(i',j)+(1-eps)*rhor*cmuoH(i',j))*m/tau+n^2*keff(i',j)*Ac/L^2;
      A(((n+1)*m+n*(j-1)+i-1)*N+(n+1)*m+n*j+i)=Nuf(i',j).*kfn(i',j)*as*Ac./((1+Bi(i',j)/5)*(2*dh))-Ac*(rhof*eps*cfn(i',j)+(1-eps)*rhor*cmuoH(i',j))*m/tau+n^2*keff(i',j)*Ac/L^2;
      A(((n+1)*m+n*(j-1)+(i-1)-1)*N+(n+1)*m+n*j+i)=-n^2*keff(i',j)*Ac/(2*L^2);
      A(((n+1)*m+n*(j)+(i-1)-1)*N+(n+1)*m+n*j+i)=-n^2*keff(i',j)*Ac/(2*L^2);
      A(((n+1)*m+n*(j-1)+(i+1)-1)*N+(n+1)*m+n*j+i)=-n^2*keff(i',j)*Ac/(2*L^2);
      A(((n+1)*m+n*(j)+(i+1)-1)*N+(n+1)*m+n*j+i)=-n^2*keff(i',j)*Ac/(2*L^2);
      i=1;
      A(((n+1)*m+n*j+i-1)*N+(n+1)*m+n*j+i)=Nuf(i',j).*kfn(i',j)*as*Ac./((1+Bi(i',j)/5)*(2*dh))+Ac*(rhof*eps*cfn(i',j)+(1-eps)*rhor*cmuoH(i',j))*m/tau+n^2*keff(i',j)*Ac/(2*L^2);;
      A(((n+1)*m+n*(j-1)+i-1)*N+(n+1)*m+n*j+i)=Nuf(i',j).*kfn(i',j)*as*Ac./((1+Bi(i',j)/5)*(2*dh))-Ac*(rhof*eps*cfn(i',j)+(1-eps)*rhor*cmuoH(i',j))*m/tau+n^2*keff(i',j)*Ac/(2*L^2);
      A(((n+1)*m+n*(j-1)+(i+1)-1)*N+(n+1)*m+n*j+i)=-n^2*keff(i',j)*Ac./(2*L^2);
      A(((n+1)*m+n*(j)+(i+1)-1)*N+(n+1)*m+n*j+i)=-n^2*keff(i',j)*Ac./(2*L^2);
      i=n;
      A(((n+1)*m+n*j+i-1)*N+(n+1)*m+n*j+i)=Nuf(i',j).*kfn(i',j)*as*Ac./((1+Bi(i',j)/5)*(2*dh))+Ac*(rhof*eps*cfn(i',j)+(1-eps)*rhor*cmuoH(i',j))*m/tau+n^2*keff(i',j)*Ac/(2*L^2);;
      A(((n+1)*m+n*(j-1)+i-1)*N+(n+1)*m+n*j+i)=Nuf(i',j).*kfn(i',j)*as*Ac./((1+Bi(i',j)/5)*(2*dh))-Ac*(rhof*eps*cfn(i',j)+(1-eps)*rhor*cmuoH(i',j))*m/tau+n^2*keff(i',j)*Ac/(2*L^2);
      A(((n+1)*m+n*(j-1)+(i-1)-1)*N+(n+1)*m+n*j+i)=-n^2*keff(i',j)*Ac./(2*L^2);
      A(((n+1)*m+n*(j)+(i-1)-1)*N+(n+1)*m+n*j+i)=-n^2*keff(i',j)*Ac./(2*L^2);
   end
   j=0;
   i=1:n;
   A(((n+1)*m+n*j+i-1)*N+(n+1)*m+n*j+i)=1;
   A(((n+1)*m+n*m+i-1)*N+(n+1)*m+n*j+i)=-1;
   X=A\B;
   for j=1:m
      i=0:n;
      Tf(:,j)=full(X((n+1)*(j-1)+i+1));
   end
   for j=0:m
      i=1:n;
     	Tr(:,j+1)=full(X((n+1)*m+n*j+i));   
   end
   err=max(max(max(abs(Tf-Tfg))),max(max(abs(Tr-Trg))))
   if(err<reltol) 
       [muf,kf,cf,rhof,hf,sf]=fluidfun(Tf);
       %[cmuoH,dsdmuoH,rhor,kr,sr]=Gd94Er6(Tr,TCurie,muoH);
       [cmuoH,dsdmuoH,rhor,kr,sr]=magpropfun(Tr,TCurie,muoH);
      done=1;
   elseif (err>reltol) && (err<1000)
      Tfg=Tf*wt+Tfg*(1-wt);
      Trg=Tr*wt+Trg*(1-wt);
   else
       Tfg=Tfl;
       Trg=Trl;
       done=1;
   end
end
for i=1:n
   for j=1:m
       if abs(mdot(j))<mdot_amp
           Tfn(i,j)=(Tr(i,j)+Tr(i,j+1))/2;
       else
           Tfn(i,j)=(Tf(i,j)+Tf(i+1,j))/2;
       end
       Trn(i,j)=(Tr(i,j)+Tr(i,j+1))/2;
   end
end
for i=1:m
    if abs(mdot(i))<mdot_amp
        Tfa(:,i)=Tfn(:,i);
    else
        Tfa(:,i)=Tfn(:,i);
    end
end
fprintf('implicit finished\n\n')


