function [dTdt]=dTdt_function(time,T,xr,Ac,as,eps,tau,Nx,L,TH,TC,massflowfun2,muoHfun2,fluidfun2,mcmpropfun2,bedcorrfun,pacCOOKED,dh,TCurie,mdot_amp,massflow_inp,muoH_max,maginp)

N=2*Nx;
[N,g]=size(T);
dTdt=zeros(N,1);

mdot=massflowfun2(time,tau,mdot_amp,massflow_inp);     %determines the mass flow of fluid vs time
[muoH,dmuoHdt]=muoHfun2(xr,time,tau,muoH_max,maginp);  %determines magnetic field vs time

i=1:Nx;
[muf,kf,cf,rhof]=fluidfun2(T(i));
v=-mdot/(Ac*rhof);
Prf=cf.*muf./kf;                %Prandtl number
Ref=dh*abs(v).*rhof./muf;       %Reynolds number based on hydraulic diameter

i=1:Nx;
[cmuoH,dsdmuoH,rhor,kr,vM_irr]=mcmpropfun2(T(Nx+i),TCurie(i),ones(Nx,1)*muoH);

[Nuf,ff,kstat,kdisp]=bedcorrfun(pacCOOKED,Ref,Prf,kf,kr,v,muf,rhof,cf,tau);
dP=ff.*(v).^2/(2*dh)*rhof*L/Nx; %Pressure drop in Pa, based on hydraulic diameter, this is correct
h=Nuf.*kf./dh;                  %convection coefficient in node i,j

if(mdot>=0)
    %hot-to-cold flow
    %fluid energy equations
    
    i=1;%hot end
    dTdt(i)=(h(i)*as*Ac*(T(Nx+i)-T(i))-mdot*cf(i)*((T(i+1)+T(i))/2-TH)*Nx/L+kdisp(i)*Ac*(T(i+1)-T(i))*Nx^2/L^2+dP(i)*abs(mdot)*Nx/L/rhof)/(rhof*Ac*eps*cf(i));
    
    for i=2:Nx-1;%internal nodes
        dTdt(i)=(h(i)*as*Ac*(T(Nx+i)-T(i))-mdot*cf(i)*(T(i+1)/2-T(i-1)/2)*Nx/L+kdisp(i)*Ac*(T(i+1)-2*T(i)+T(i-1))*Nx^2/L^2+dP(i)*abs(mdot)*Nx/L/rhof)/(rhof*Ac*eps*cf(i));
    end
    
    i=Nx;%cold end
    dTdt(i)=(h(i)*as*Ac*(T(Nx+i)-T(i))-mdot*cf(i)*(T(i)-T(i-1))*Nx/L+kdisp(i)*Ac*(T(i-1)-T(i))*Nx^2/L^2+dP(i)*abs(mdot)*Nx/L/rhof)/(rhof*Ac*eps*cf(i));
    
else
    if(mdot<0)
        %cold-to-hot flow
        %fluid energy equations
        
        i=1;%hot end
        dTdt(i)=(h(i)*as*Ac*(T(Nx+i)-T(i))-mdot*cf(i)*(T(i+1)-T(i))*Nx/L+kdisp(i)*Ac*(T(i+1)-T(i))*Nx^2/L^2+dP(i)*abs(mdot)*Nx/L/rhof)/(rhof*Ac*eps*cf(i));
        
        for i=2:Nx-1;%internal nodes
            dTdt(i)=(h(i)*as*Ac*(T(Nx+i)-T(i))-mdot*cf(i)*(T(i+1)/2-T(i-1)/2)*Nx/L+kdisp(i)*Ac*(T(i+1)-2*T(i)+T(i-1))*Nx^2/L^2+dP(i)*abs(mdot)*Nx/L/rhof)/(rhof*Ac*eps*cf(i));
        end
        
        i=Nx;%cold end
        dTdt(i)=(h(i)*as*Ac*(T(Nx+i)-T(i))-mdot*cf(i)*(TC-(T(i-1)+T(i))/2)*Nx/L+kdisp(i)*Ac*(T(i-1)-T(i))*Nx^2/L^2+dP(i)*abs(mdot)*Nx/L/rhof)/(rhof*Ac*eps*cf(i));
        
    else
        %no flow
    end
end

%regenerator energy equations

i=1;%hot end
dTdt(Nx+i)=((h(i)*as*Ac*(T(i)-T(Nx+i))+kstat(i)*Ac*(T(Nx+i+1)-T(Nx+i))*Nx^2/L^2)/(rhor(i)*Ac*(1-eps))-T(Nx+i)*dsdmuoH(i)*dmuoHdt+vM_irr(i)*abs(dmuoHdt))/cmuoH(i);

for i=2:Nx-1%internal nodes
    dTdt(Nx+i)=((h(i)*as*Ac*(T(i)-T(Nx+i))+kstat(i)*Ac*(T(Nx+i+1)-2*T(Nx+i)+T(Nx+i-1))*Nx^2/L^2)/(rhor(i)*Ac*(1-eps))-T(Nx+i)*dsdmuoH(i)*dmuoHdt+vM_irr(i)*abs(dmuoHdt))/cmuoH(i);
end

i=Nx;%cold end
dTdt(Nx+i)=((h(i)*as*Ac*(T(i)-T(Nx+i))+kstat(i)*Ac*(T(Nx+i-1)-T(Nx+i))*Nx^2/L^2)/(rhor(i)*Ac*(1-eps))-T(Nx+i)*dsdmuoH(i)*dmuoHdt+vM_irr(i)*abs(dmuoHdt))/cmuoH(i);

end