%% This function calculates model parameters for several packing geometries
%% It accepts an input array named inputs and calculates the appropriate
%% hydraulic diameter, porosity and specific surface area.  The array
%% 'inputs' contains different information about the regenerator geometry
%% for each type of regenerator.  The order of variables in 'inputs' is  
%%
%% For a packed sphere regenerator: 1. volumetric porosity
%%      2. dummy variable
%%      3. particle diameter
%%      4. dummy variable
%% 
%% For parallel plate regenerators: 1. Dx, plate thickness in meters
%%      2. Px, plate spacing (from plate to plate, not flow channel length)
%%      in meters
%%      3. dummy variable
%%      4. dummy variable

function[dh,eps,as]=pack_geom(inputs,bedcorrfun);
 
geometry=func2str(bedcorrfun);

if (strcmp(geometry,'SphPart2')==1 | strcmp(geometry,'SphPart_SB_2par')==1 | strcmp(geometry,'SphPart_SB_1par')==1 | strcmp(geometry,'SphPart2_old3')==1)%spherical particle geometryt

    %% assign variable to elements in the inputs array

    eps=inputs(1);
    dp=inputs(3);

    %% calculate geometry-specific parameters

    dh=2/3*eps/(1-eps)*dp; %m
    as=6*(1-eps)/dp; %m^2/m^3

elseif(strcmp(geometry,'AlignRect')==1 | strcmp(geometry,'AlignRect_ha')==1)%aligned rectangular plater regenerator

    %%parse the input variable

    Dx=inputs(1);
    Px=inputs(2);
    %processing...
    as=2/Px;
    eps=1-Dx/Px;
    dh=4*eps/as;

else
    error('geometry not yet coded or mismatch error with geometry and inputs');
end

   